// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
		"cloudera/chart/ServiceChartSet",
		"cloudera/chart/TimeRange",
		"cloudera/Util",
		"cloudera/chart/lang/AllLocales",
		"underscore"
], function(ServiceChartSet, TimeRange, Util, AllLocales, _) {

	return ServiceChartSet.extend({

		_preinitialize: function() {
			this._super.apply(this, arguments);
			this.state = this.options.state;
			this.allowPadding = true;
			this.allowChartsToStretch = false;
		},

		// On the time control, there is one master time series.
		getDefaultDataSetId: function() {
			return this.getDataSetId(this.options.timeSeriesDescriptors[0]);
		},

		getDefaultDataProviderId: function() {
			return "dp_0";
		},

		createChart: function(chartId, chartTitle) {
			var chart = {
				'topPadding': 0,
				'bottomPadding': 0,
				'enabled':false,
				'id': chartId,
				'height': 1,
				'legend': {
					'enabled': false,
					'title': {
						'enabled': false
					},
					'background':{
						'enabled':true,
						'opacity':1,
						'fill':{
							'color':'#FFFFFF',
							'enabled':true,
							'type':'Solid'
						}
					}
				},
				'xAxis':{
					'enabled': false
				},
				'valueAxes':{
					'primary': {
						'enabled': true,
						'scale': {
							'minimumMode': 'CustomValue',
							'minimum': 0
						},
						'line': {
							'enabled':false
						},
						'grid': {
							'enabled':false
						},
						'zeroLine': {
							'enabled':false
						}
					}
				},
				'seriesList': this.createSeriesList()
			};
			return chart;
		},

		getChartType: function(chart) {
			return "Line";
		},

		getColor: function(timeSeriesDescriptor, index) {
			return "#ffffff";
		},

		createSettings: function(){
			// Note the outsideMargin's right width
			// is markerRightWidth so that
			// the marker can be moved all the way
			// so that its ponter can point to the right
			// hand side of the scroller.
			var localeSettings = AllLocales.getLocale(Util.getLocale()).settings;
			var settings = {
				'dataGrouping':{
					'enabled':false
				},
				'legend': {
					'enabled':false
				},
				'outsideMargin': {
					'left': 0,
					'top': 0,
					'right': this.state.markerRightWidth,
					'bottom': 0
				},
				'insideMargin': {
					'all': 0
				},
				'charts': this.createCharts(),
				'scroller': {
					'height': 40,
					'enabled': true,
					'dataProvider': this.getDefaultDataProviderId(),
					'connectMissingPoints': false,
					'insideSelectedRange': this.getInsideSelectedRange(),
					'outsideSelectedRange': this.getOutsideSelectedRange(),
					'rangeMarkers': {
						'enabled': this.isRangeInteractive(),
						'showAlways': this.isRangeInteractive()
					},
					'minimumMode': 'CustomValue',
					'minimum': 0,
					'maximumMode': 'CustomValue',
					'maximum': 100,
					'timeScale': {
						'intervalFormats': localeSettings.timeScale.intervalFormats
					}
				},
				'timeScale': {
					'isOrdinal':false,
					'selectedRange': {
						'type':'Max'
					}
				},
				'displayLocale': localeSettings.displayLocale
			};
			return settings;
		},

		getInsideSelectedRange: function() {
			if (this.isRangeInteractive()) {
				return {
					'seriesFill': {
						'color': '#54ACDC',
						'enabled': true,
						'type': 'Gradient',
						'gradient': {
							'type': 'Radial',
							'focal_point': -0.5,
							'angle': 45,
							'keys': [ {
								'position': 0,
								'color': '#53ABDB'
							}, {
								'position':1,
								'color': '#53ABDB'
							}]
						}
					},
					'backgroundFill': {
						'color': '#d3f5fe',
						'enabled': true
					}
				};
			} else {
				return this.getOutsideSelectedRange();
			}
		},

		getOutsideSelectedRange: function() {
			return {
				'seriesFill': {
					'color': '#999999',
					'enabled': true,
					'type': 'Gradient',
					'gradient': {
						'type': 'Linear',
						'focal_point': -0.5,
						'angle': 45,
						'keys': [ {
							'position': 0,
							'color': '#999999'
						}, {
							'position':1,
							'color': '#dddddd'
						}]
					}
				},
				'backgroundFill': {
					'color': '#ffffff',
					'enabled': true,
					'type': 'Solid'
				}
			};
		},

		getExpandFactor: function() {
			return 1;
		},

		// This represents how much data we need to prefetch.
		// A factor of 3 means the selected range of the time control
		// is 1/3rd of the total time period.
		getCacheFactor: function() {
			return 3;
		},

		_broadcastRangeChange: function() {
			var self = this;
			// if we are in current mode, dragging the range means we are off the current mode.
			// if we are not in current mode, dragging the range should not make us go into current mode.
			if (self.state.getCurrentMode()) {
				self.state.setCurrentMode(self.state.isDateCurrent(self.lastVisibleDate));
			}
			self.state.setSelectedRange(new TimeRange(self.firstVisibleDate, self.lastVisibleDate));
			self.state.moveMarkerIntoSelectedRange();
		},

		onSelectedRangeChange: function(chartStartDate, chartEndDate, phase, sender) {
			var self = this;

			if (sender === "scroller" && phase === "finish") {
				this._super.apply(this, arguments);
				_.debounce(function() {
					self._broadcastRangeChange();
				}, 500);
			}
		},

		removeAllData: function() {
			var dataSetId = this.getDefaultDataSetId();
			// Completely remove everything in the old range,
			// even if the startDate and the endDate is within the current range.
			// Note we have to use the date range from the chart, not from
			// this.firstDate and this.lastDate, which is due to the timezone offset from GMT.
			this.chart.removeDataRange(dataSetId, this.chart.getFirstDate(), this.chart.getLastDate());
		},

		zoomTo: function(startDate, endDate) {
			if (this.isRangeInteractive()) {
				this._super.apply(this, arguments);
				this._broadcastRangeChange();
			}
		},

		// Overrides the default time initializer in the base class.
		initializeTime: function(){
			var visibleRange = new TimeRange(this.state.firstVisibleDate, this.state.lastVisibleDate);
			this._setVisibleRange(visibleRange);

			var cachedRange = new TimeRange(this.state.firstDate, this.state.lastDate);
			this._setCachedRange(cachedRange);

			this.logTime();
		},

		// Specify the data set.
		initializeData: function() {
			this._super.apply(this, arguments);

			var timeSeriesDescriptor = this.options.timeSeriesDescriptors[0];
			this.addTimeSeries(timeSeriesDescriptor);

			this.getScrollerDataProviders().push({
				'id': this.getDataProviderId(timeSeriesDescriptor),
				'dataSet': this.getDefaultDataSetId(),
				'column': 1
			});

			var self = this;
			var onChartRangeChanged = function(range){
				// the chart's range has changed, update my highlighted selection.
				if (self.state.getCurrentMode()) {
					self.state.setCurrentMode(self.state.isDateCurrent(range.endDate));
				}
				self.state.setSelectedRange(range);
				self.state.moveMarkerIntoSelectedRange();
			};
			var handle1 = jQuery.subscribe("chartRangeChanged", onChartRangeChanged);

			// Update the selected time range.
			var onTimeSelectionChanged = function(range, isCurrentMode) {
				self.firstVisibleDate = range.startDate;
				self.lastVisibleDate = range.endDate;
				self.zoomTo(self.firstVisibleDate, self.lastVisibleDate);
			};
			var handle2 = jQuery.subscribe("timeSelectionChanged", onTimeSelectionChanged);

			// Update the total time range.
			var onTotalTimeRangeChanged = function(range, isCurrentMode) {
				self.removeAllData();
				self.firstDate = range.startDate;
				self.lastDate = range.endDate;
				self._checkRange(true, self.state.getCurrentMode());
			};
			var handle3 = jQuery.subscribe("totalTimeRangeChanged", onTotalTimeRangeChanged);

			self.subscriptionHandles = [handle1, handle2, handle3];
		},

		_setCachedRange: function(range) {
			this._super.apply(this, arguments);
			this.state.setTotalRange(range);
		},

		onFetchError: function(data, startDate, endDate) {
			// We have a fetch error.
			var defaultDataSetId = this.getDefaultDataSetId();
			this.chart.appendData(defaultDataSetId, String(this.firstDate.getTime()) + ",0\n" + this.lastDate.getTime() + ",0");
			this.chart.commitDataChanges();
			this.onFetchedMoreData();
		},

		onFetchedMoreData: function() {
			this._super.apply(this, arguments);
		},

		// Whether the chart is interactive or not.
		isRangeInteractive: function() {
			return (this.options.mode === "INTERACTIVE" && this.options.showRange);
		},

		onChartDraw: function() {
			this._super.apply(this, arguments);

			if (this.isRangeInteractive()) {
				this.chart.enableInteractivity();
			} else {
				this.chart.disableInteractivity();
			}
			this.zoomTo(this.firstVisibleDate, this.lastVisibleDate);
		},

		unsubscribe: function() {
			Util.unsubscribe(this);
		}
	});

});
