// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/Util",
	"cloudera/common/Humanize"
], function(Util, Humanize) {

describe("Humanize Tests", function() {

	it("should test humanizeBytes.", function() {
		expect(Humanize.humanizeBytes(13)).toEqual("13 B");
		expect(Humanize.humanizeBytes(1024)).toEqual("1.0 KiB");
		expect(Humanize.humanizeBytes(1024 + 512)).toEqual("1.5 KiB");
		expect(Humanize.humanizeBytes(5 * 1024 * 1024 * 1024 * 1024 * 1024))
				.toEqual("5.0 PiB");
		expect(Humanize.humanizeBytes(2000 * 1024 * 1024 * 1024 * 1024 * 1024))
				.toEqual("2.0 EiB");
		expect(
				Humanize.humanizeBytes(-1 * 2000 * 1024 * 1024 * 1024 * 1024
						* 1024)).toEqual("-2.0 EiB");
	});

	it("should test humanizeSeconds.", function() {
		expect(Humanize.humanizeSeconds(13)).toEqual("13.00s");
		expect(Humanize.humanizeSeconds(120)).toEqual("2.0m");
		expect(Humanize.humanizeSeconds(60 * 60 * 2)).toEqual("2.0h");
		expect(Humanize.humanizeSeconds(60 * 60 * 24 * 2)).toEqual("2.0d");
	});

	it("should test humanizeTimeShortAndMS", function() {
		var d = new Date(100101);
		expect(Humanize.humanizeTimeShortAndMS(d)).toEqual("4:01:40.101 PM");
	});

	it("should test humanizeMilliSeconds.", function() {
		expect(Humanize.humanizeMilliseconds(13)).toEqual("0.01s");
		expect(Humanize.humanizeMilliseconds(1234)).toEqual("1.23s");
	});

	it("should test timezone offset.", function() {
		var oldOffset = Util.getServerTimezoneOffset();
		var MS_IN_ONE_MINUTE = 60 * 1000;
		var MS_IN_ONE_HOUR = 60 * MS_IN_ONE_MINUTE;

		Util.setServerTimezoneOffset(oldOffset - MS_IN_ONE_HOUR);
		var serverDate = new Date();
		var localDate = Humanize.fromServerDate(serverDate);
		expect(localDate.getTime() - serverDate.getTime()).toEqual(MS_IN_ONE_HOUR);

		var serverDate2 = Humanize.toServerDate(localDate);
		expect(serverDate2.getTime()).toEqual(serverDate.getTime());
		expect(localDate.getTime() - serverDate2.getTime()).toEqual(MS_IN_ONE_HOUR);
		Util.setServerTimezoneOffset(oldOffset);
	});
});
});
